
/* Bear Transfer Protocol (BTP), a TCP/IP socket based file 
   transfer utility

  The author of this software is Ian Kaplan
  www.bearcave.com
  iank@bearcave.com

  Copyright (c) Ian Kaplan, 1999

  See "copyright" file for allowed use.

*/

#include "netinc.h"
#include "comline.h"

/*
 *         Client Functions
 *
 */


void command_line_client::usage(void) { 
  fprintf(stderr,"Usage\n%s -p <port> <system>\n", get_prog_name()); 
  fprintf(stderr,"Where:\n\t<system> is a system name (e.g., foobar.com)\n");
  fprintf(stderr,"    or\n\t<system> is an IP address (e.g., 172.0.0.1)\n");
  fprintf(stderr,"\t<port> is port to send data on.\n");
  fprintf(stderr,"Default port is 5001\n");
#ifdef WIN32
  WSACleanup(); 
#endif
  exit(1); 
} 


/*
 * is_ip_addr
 *

   Return TRUE if the string argument is an IP address (e.g,.
   a string composed of nothing but numbers and at least three
   periods (not that the new long IP addresses are 6 bytes).

 */
Boolean command_line_client::is_ip_addr( const char *ip_addr )
{
  Boolean retval;

  retval = FALSE;
  if (ip_addr != NULL) {
    int period_cnt;

    retval = TRUE;
    period_cnt = 0;
    while (*ip_addr != '\0') {

      if (*ip_addr == '.')
	period_cnt++;
      else if (! (isdigit( *ip_addr ) )) {
	retval = FALSE;
	break;
      }

      ip_addr++;
    } // while
    if (period_cnt != 3 || period_cnt != 5)
      retval = FALSE;
  }
  return retval;
} // is_ip_addr


/*

   Args:
     - program name
     -p <port number>
     <system>

 */
void command_line_client::parse_comm_line( int argc, char *argv[] )
{
  const int MAX_ARGS = 5;
  int i;
  char *pArg;
  
  if (argc > 1 && argc <= MAX_ARGS ) { 
    for(i = 1; i < argc; i++) { 

      pArg = argv[i];
      
      if ( (pArg[0] == '-') || (pArg[0] == '/') ) { 
	switch( tolower( pArg[1] ) ) { 

       // port argument
       case 'p': 
	 if (i+1 < argc) {
	   i++;
	   set_port(atoi(argv[i]));
	 }
	  break; 

	default: 
	  usage(); 
	  break; 
       } // switch
      }
      else if (isdigit(pArg[0])) { // its an IP address 123.45.67.89
	if (is_ip_addr( argv[i] ))
	  put_ip( argv[i] );
	else
	  put_name( argv[i] );
      }
      else {
	// it must be a system name (e.g., foobar.com)
	put_name( argv[i] );
      }
    } 
  } 
  else
    usage();

} // parse_comm_line



/*
 *         Server Functions
 *
 */


void command_line_server::usage(void) { 
  fprintf(stderr,"Usage\n%s -p [port]\n", get_prog_name()); 
  fprintf(stderr,"Where:\n\tport is the port to listen on\n"); 
  fprintf(stderr,"\tex: %s -p 6543\n", get_prog_name() ); 
  fprintf(stderr,"Default port is 5001\n");
#ifdef WIN32
  WSACleanup(); 
#endif
  exit(1); 
} 


void command_line_server::parse_comm_line( int argc, char *argv[] )
{
  const int MAX_ARGS = 2;
  int i;
  char *pArg;
  
  if (argc > 1 && argc <= MAX_ARGS ) { 
    for(i = 1; i < argc; i++) { 

      pArg = argv[i];
      
      if ( (pArg[0] == '-') || (pArg[0] == '/') ) { 
	switch( tolower( pArg[1] ) ) { 

       // port argument
       case 'p': 
	 if (i+1 < argc) {
	   i++;
	   set_port(atoi(argv[i]));
	 }
	  break; 

	default: 
	  usage(); 
	  break; 
       } // switch
      } 
      else 
	usage(); 
    } 
  } 

} // parse_comm_line
