
/* Bear Transfer Protocol (BTP), a TCP/IP socket based file 
   transfer utility

  The author of this software is Ian Kaplan
  www.bearcave.com
  iank@bearcave.com

  Copyright (c) Ian Kaplan, 1999

  See "copyright" file for allowed use.

*/

#include "netinc.h"
#include "heapsort.h"
#include "array.h"
#include "dirnames.h"



void dirnames::add_dir_name( char *name )
{
  size_t num, len;
  
  assert( name != NULL );
  
  names.append( name );
  len = strlen( name );
  if (len > max_len)
    max_len = len;
  
  num = names.get_size();
  if (num > 1) {
    if (names[num-2] > names[num-1]) {
      sorted = FALSE;
    }
  }
} // add_dir_name



/*
 * print_names
 *

   List the directory names in format modeled after UNIX ls.

 */
void dirnames::print_names()
{
  const size_t line_size = 79;
  size_t num_cols = 4;
  size_t word_len;
  size_t col;
  size_t col_width;
  char col_format[20];
  int i, j;
  int num_names, max_in_col;

  num_names = names.get_size();
  if (num_names > 0) {

    if (! is_sorted() ) {
      sort();
    }

    // figure out how many columns to print the file names in
    word_len = get_max_len();
    col = line_size / word_len;
    if (col == 0)
      num_cols = 1;
    else if (col < num_cols)
      num_cols = col;

  // width of a column
    col_width = line_size / num_cols;

    // build the format string to use in printing
    // the directory entries
    sprintf(col_format, "%%-%ds", col_width );

    max_in_col = (num_names / num_cols) + (int)((num_names % num_cols) > 0);

    for (i = 0; i < max_in_col; i++) {
      for (j = i; j < num_names; j = j + max_in_col) {
	printf(col_format, names[j] );      
      }
      printf("\n");
    }
  }
} // print_names
