
#ifndef _DIRNAMES_H_
#define _DIRNAMES_H_


/* Bear Transfer Protocol (BTP), a TCP/IP socket based file 
   transfer utility

  The author of this software is Ian Kaplan
  www.bearcave.com
  iank@bearcave.com

  Copyright (c) Ian Kaplan, 1999

  See "copyright" file for allowed use.

*/


/*
 * dirnames
 *

   A class to support a sorted list of directory names.  At least on
   NT 4.0 the names in a directory come in sorted order.  For UNIX
   this is not true.  When names are loaded into the GrowableArray the
   class check to see if they are sorted.  If they are not the class
   function is_sorted() will return FALSE.  The sort() function will
   sort the array.

 */

class dirnames : public heapsort {
private:

  /*
   * Implementations of virtual functions in the
   * heapsort parent class.
   */
  int lessthan( size_t left, size_t right )
  {
    int rslt, array_size;

    array_size = names.get_size();

    assert( left < array_size );
    assert( right < array_size );

    rslt = (strcmp(names[left], names[right]) < 0);

    return rslt;
  }
  
  void exchange( size_t left, size_t right ) 
  {
    int array_size;
    char *tmp;

    array_size = names.get_size();

    assert( left < array_size );
    assert( right < array_size );
    
    tmp = names[left];
    names[left] = names[right];
    names[right] = tmp;
  }

  size_t get_size() { return names.get_size(); }

  size_t get_max_len(void) { return max_len; }

  Boolean is_sorted(void) { return sorted; }

private:
  Boolean sorted;  // TRUE if array is in increasing order
  size_t max_len;  // maximum name lenght

public:
  GrowableArray<char *> names;

  dirnames( void *(*GetMem)(const unsigned int size ),
		 void (*FreeMem)( void *) ) : names( GetMem, FreeMem ) 
  {
    sorted = TRUE;
    max_len = 0;
  }

  void add_dir_name( char *name );

  void print_names(void);

}; // dirnames

#endif
