
#ifndef _HEAPSORT_H_
#define _HEAPSORT_H_


/* Bear Transfer Protocol (BTP), a TCP/IP socket based file 
   transfer utility

  The author of this software is Ian Kaplan
  www.bearcave.com
  iank@bearcave.com

  Copyright (c) Ian Kaplan, 1999

  See "copyright" file for allowed use.

*/


/*
 * Generic heapsort class
 *

   The heapsort class is the base class, which should be used to
   derive a class that will sort an array of elements of any type.
   The derived class should contain or have access to the array
   to be sorted.

   The derived class implements the functions:

     - lessthan
   
       This function returns 1 if the array element A[left] is
       less than the array element A[right].

     - exchange

       Exchange A[left] with A[right]

     - get_size

       Return the number of elements in the array.

   Example:

    A string sort class, derived from heapsort.  Note that the
    strsort class contains the array that is being sorted.

    class strsort : public heapsort {

      int lessthan( size_t left, size_t right )
      {
        int rslt;

	rslt = (strcmp(names[left], names[right]) < 0);

	return rslt;
      }
  
      void exchange( size_t left, size_t right ) 
      {
        char *tmp;

	tmp = names[left];
	names[left] = names[right];
	names[right] = tmp;
      }

      size_t get_size() { return size }

    public:
      char * names[ MAX_NAMES ];
  };

 */

class heapsort {
private:
  void sift( size_t left, size_t size );

protected:
  // compare two elements at indices "left" and "right" and
  // return TRUE if a[left] < a[right]
  virtual int lessthan( size_t left, size_t right )
  {
    assert( FALSE );
    return FALSE;
  }

  // exchante two elements at indices "left" and "right"
  virtual void exchange( size_t left, size_t right )
  {
    assert( FALSE );
  }

  // return the size of the array to be sorted
  virtual size_t get_size()
  {
    assert( FALSE );
    return 0;
  }

public:
  heapsort() {}
  void sort(void);

}; // heapsort

#endif
