
/* Bear Transfer Protocol (BTP), a TCP/IP socket based file 
   transfer utility

  The author of this software is Ian Kaplan
  www.bearcave.com
  iank@bearcave.com

  Copyright (c) Ian Kaplan, 1999

  See "copyright" file for allowed use.

*/


#include "netinc.h"
#include "neterr.h"


/*
 * PrintErr
 *

   Print an error message, but don't exit
 
 */
void
neterr::PrintErr( char *msg )
{
  if (msg != NULL) {
    fprintf(stderr, "%s", msg );
  }

#ifdef WIN32
  fprintf(stderr, " %d, %s", WSAGetLastError(), strerror(WSAGetLastError()) );
#else
  fprintf(stderr, " %s", strerror( errno ) );
#endif
  fprintf(stderr, "\n");

} // PrintErr



/*
 * Error
 *

   Report a serious error and exit the program with
   the status -1.

 */
void 
neterr::Error(char *msg)
{
  PrintErr( msg );

#ifdef WIN32
    WSACleanup(); 
#endif

  exit( -1 ); 
} /* Error */



/*
 * get_errno
 *
   System independent (UNIX or Win32) function to get the
   error number.

 */
int
neterr::get_errno()
{
  int err_ret;

#ifdef WIN32
  err_ret = WSAGetLastError();
#else
  err_ret = errno;
#endif
  
  return err_ret;
} /* get_errno */



/*
  Determine whether or not the error result is EINTR/WSAEINTR.
  This indicates that the socket read (recv) operation has been
  interruped and should be retried.  The Dark Lord knows whether
  this actually works under Win32.

  Definition for:

      WSAEINTR (win32)
      EINTR    (unix)

      Interrupted function call. An asynchronous signal (such as
      SIGINT or SIGQUIT) was caught by the process during the
      execution of an interruptible function. If the signal handler
      performs a normal return, the interrupted function call will
      seem to have returned the error condition.

 */
int
neterr::is_EINTR( int err_rslt )
{
  int rslt;

  rslt = 0;

  if (err_rslt != 0) {
#ifdef WIN32
    rslt = (err_rslt == WSAEINTR);
#else
    rslt = (err_rslt == EINTR);
#endif
  }

  return rslt;
} /* is_EINTR */
