
#ifndef _NETIO_H_
#define _NETIO_H_


/* Bear Transfer Protocol (BTP), a TCP/IP socket based file 
   transfer utility

  The author of this software is Ian Kaplan
  www.bearcave.com
  iank@bearcave.com

  Copyright (c) Ian Kaplan, 1999

  See "copyright" file for allowed use.

*/


/*

  netio class

  The netio class is the base class for TCP I/O and for
  file I/O.

  The class contains two static variables which will be
  shared by all instances of the class:
  

  mismatch:

    This variable is true if there is an "endian" mismatch between the
    client and the server.

  mode

    This variable will be either ascii_mode or binary_mode.  On a UNIX
    to UNIX transfer, this variable will have no effect, since binary
    mode is ignored on UNIX systems.  On a transfer between a UNIX 
    system and an NT system, binary mode stops translation of 
    Windows CR NL sequences into a NL.  Binary mode must be used
    for the transfer of tar files and other binary data.

  These variables are initialized in globals.C as:

    Boolean  netio::mismatch = FALSE;
    COMMANDS netio::mode = ascii_mode;


 */

#define Kilo         1024
#define IO_BUF_EXTRA (sizeof(COMMANDS) + sizeof(int))
#define IO_BUF_SIZE  (8 * Kilo) - IO_BUF_EXTRA


typedef struct {
  COMMANDS com;
  unsigned int size;
  char buf[ IO_BUF_SIZE ];
} io_block;


class netio : protected neterr 
{
private:
  static Boolean mismatch;  // "endian" mismatch between client and server
  static COMMANDS mode; // ascii_mode (default) or binary_mode

private:
  Boolean debug;

private:
  int get_char(int fd, char *pCh );
  int readline(int fd, void *vptr, size_t maxlen);

protected:
  typedef enum { DATA_BUF_SIZE = 1024 } bogosity;
  unsigned int byte_flip( const unsigned int little );

public:
  netio()
  {
    debug = FALSE;
  }
  int writen(int fd, const void *vptr, size_t n);
  int read_block(int fd, char *pBlock, unsigned int block_size );
  void Writen(int fd, void *ptr, size_t nbytes);
  void send_io_block( int sockfd, io_block &blk, int num_bytes);
  void SendCommand(int sockfd, COMMANDS com );
  void GetAck( int sockfd, COMMANDS com, const char *prog_name );
  COMMANDS RecvCommand( int sockfd, const char *prog_name );
  void client_endian_check(int sockfd );
  void server_endian_check( int sockfd );
  Boolean endian_mismatch(void) { return mismatch; }
  COMMANDS get_mode(void) { return mode; }
  void set_mode( COMMANDS m ) { mode = m; }
  void print_mode(void);
  void set_debug()   { debug = TRUE; }
  void clear_debug() { debug = FALSE; }
}; // netio


#endif
