
/* Bear Transfer Protocol (BTP), a TCP/IP socket based file 
   transfer utility

  The author of this software is Ian Kaplan
  www.bearcave.com
  iank@bearcave.com

  Copyright (c) Ian Kaplan, 1999

  See "copyright" file for allowed use.

*/


#include "netinc.h"
#include "neterr.h"
#include "netwrap.h"


/*
 * Socket
 *
 */
SOCKET 
netwrap::Socket(int family, int type, int protocol)
{
  SOCKET  conn_socket; 

  conn_socket = socket(family, type, protocol ); /* Open a socket */ 

  if (conn_socket < 0 ) { 
    Error("Server: Error Opening socket: Error");
  } 

  return conn_socket;
}  // Socket



void 
netwrap::Bind( SOCKET listen_socket, struct sockaddr *pSockAddr, int data_size)
{
  // 
  // bind() associates a local address and port combination with the 
  // socket just created. This is most useful when the application is a  
  // server that has a well-known port that clients know about in advance. 
  // 
  
  if (bind(listen_socket, pSockAddr, data_size ) == SOCKET_ERROR) { 
    Error( "bind() failed with error" ); 
  } 
} // Bind



void 
netwrap::Listen( SOCKET listen_socket, int backlog )
{
  if (listen(listen_socket, backlog) == SOCKET_ERROR) {
    Error( "listen() failed with error" );
  } 
} // Listen


SOCKET 
netwrap::Accept( SOCKET listen_socket, struct sockaddr *pFrom, int *pFrom_size )
{
  SOCKET msgsock;
  
  msgsock = accept(listen_socket, pFrom, pFrom_size );

  if (msgsock == INVALID_SOCKET) { 
    Error( "accept() error"); 
  } 

  return msgsock;
} // Accept


/*
 * Connect
 *
 */
void 
netwrap::Connect( SOCKET conn_socket, struct sockaddr *pServer, 
		  size_t data_size ) 
{
  assert( pServer != NULL );

  if (connect(conn_socket, pServer, data_size ) == SOCKET_ERROR) { 
    Error("connect() failed"); 
  } 
} // Connect

